import { IABIMethod } from "./ethjs-abi";
import { ContractSendReceipt } from "./ContractSendReceipt";
import { ICallContractResult, IExecutionResult, ISendToContractResult, QtumRPC } from "./QtumRPC";
export interface IContractInfo {
    /**
     * Contract ABI methods
     */
    abi: IABIMethod[];
    /**
     * Address of contract
     */
    address: string;
}
export interface IContractCallResult extends IExecutionResult {
    [key: number]: any;
}
export declare class Contract {
    private rpc;
    private address;
    private callMethodsMap;
    private sendMethodsMap;
    constructor(rpc: QtumRPC, info: IContractInfo);
    encodeParams(method: string, args?: any[]): string;
    /**
     * Call a contract method using ABI encoding, and return the RPC result as is. This
     * does not create a transaction. It is useful for gas estimation or getting results from
     * read-only methods.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    rawCall(method: string, args?: any[], opts?: {}): Promise<ICallContractResult>;
    call(method: string, args?: any[], opts?: {}): Promise<IContractCallResult>;
    /**
     * Create a transaction that calls a method using ABI encoding, and return the RPC result as is.
     * A transaction will require network consensus to confirm, and costs you gas.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    rawSend(method: string, args: any[], opts?: {}): Promise<ISendToContractResult>;
    send(method: string, args: any[], opts?: {}): Promise<ContractSendReceipt>;
}
