"use strict";
// import {
//   decodeParams,
//   encodeMethod,
//   IABI,
// } from
Object.defineProperty(exports, "__esModule", { value: true });
// const {
//   decodeParams,
//   encodeMethod,
// } = require("ethjs-abi") as IETHABI
const abi_1 = require("./abi");
const ContractSendReceipt_1 = require("./ContractSendReceipt");
class Contract {
    constructor(rpc, info) {
        this.rpc = rpc;
        this.callMethodsMap = {};
        this.sendMethodsMap = {};
        for (const methodABI of info.abi) {
            const name = methodABI.name;
            // Allow sendToContract only for non-constant methods
            if (!methodABI.constant) {
                this.sendMethodsMap[name] = methodABI;
            }
            this.callMethodsMap[name] = methodABI;
        }
        this.address = info.address;
    }
    encodeParams(method, args = []) {
        const methodABI = this.callMethodsMap[method];
        if (methodABI == null) {
            throw new Error(`Unknown method to call: ${method}`);
        }
        return abi_1.encodeInputs(methodABI, args);
    }
    /**
     * Call a contract method using ABI encoding, and return the RPC result as is. This
     * does not create a transaction. It is useful for gas estimation or getting results from
     * read-only methods.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    async rawCall(method, args = [], opts = {}) {
        // TODO opts: sender address
        // need to strip the leading "0x"
        const calldata = this.encodeParams(method, args);
        // TODO decode?
        return this.rpc.callContrct({
            address: this.address,
            datahex: calldata,
        });
    }
    async call(method, args = [], opts = {}) {
        // TODO support the named return values mechanism for decodeParams
        const r = await this.rawCall(method, args, opts);
        const exception = r.executionResult.excepted;
        if (exception !== "None") {
            throw new Error(`Call exception: ${exception}`);
        }
        const output = r.executionResult.output;
        if (output === "") {
            return r.executionResult;
        }
        const methodABI = this.callMethodsMap[method];
        const returnValues = abi_1.decodeOutputs(methodABI, output);
        return Object.assign(returnValues, r.executionResult);
    }
    /**
     * Create a transaction that calls a method using ABI encoding, and return the RPC result as is.
     * A transaction will require network consensus to confirm, and costs you gas.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    async rawSend(method, args, opts = {}) {
        // TODO opts: gas limit, gas price, sender address
        const methodABI = this.sendMethodsMap[method];
        if (methodABI == null) {
            throw new Error(`Unknown method to send: ${method}`);
        }
        const calldata = abi_1.encodeInputs(methodABI, args);
        return this.rpc.sendToContract({
            address: this.address,
            datahex: calldata,
        });
    }
    async send(method, args, opts = {}) {
        const r = await this.rawSend(method, args, opts);
        return new ContractSendReceipt_1.ContractSendReceipt(this.rpc, r);
    }
}
exports.Contract = Contract;
