import { QtumRPCRaw } from "./QtumRPCRaw";
export interface IGetInfoResult {
    version: number;
    protocolversion: number;
    walletversion: number;
    balance: number;
    stake: number;
    blocks: number;
    timeoffset: number;
    connections: number;
    proxy: string;
    difficulty: {
        "proof-of-work": number;
        "proof-of-stake": number;
    };
    testnet: boolean;
    moneysupply: number;
    keypoololdest: number;
    keypoolsize: number;
    paytxfee: number;
    relayfee: number;
    errors: string;
}
export interface ISendToContractRequest {
    /**
     * (required) The contract address that will receive the funds and data.
     */
    address: string;
    /**
     * (required) data to send
     */
    datahex: string;
    /**
     * The amount in QTUM to send. eg 0.1, default: 0
     */
    amount?: number | string;
    /**
     * gasLimit, default: 200000, max: 40000000
     */
    gasLimit?: number;
    /**
     * Qtum price per gas unit, default: 0.00000001, min:0.00000001
     */
    gasPrice?: number | string;
    /**
     * The quantum address that will be used as sender.
     */
    senderAddress?: string;
}
export interface ISendToContractResult {
    /**
     * The transaction id.
     */
    txid: string;
    /**
     * QTUM address of the sender.
     */
    sender: string;
    /**
     * ripemd-160 hash of the sender.
     */
    hash160: string;
}
export interface ICallContractRequest {
    /**
     * (required) The account address
     */
    address: string;
    /**
     * (required) The data hex string
     */
    datahex: string;
    /**
     * The sender address hex string
     */
    senderAddress?: string;
}
export interface IExecutionResult {
    gasUsed: number;
    excepted: string;
    newAddress: string;
    output: string;
    codeDeposit: number;
    gasRefunded: number;
    depositSize: number;
    gasForDeposit: number;
}
export interface ICallContractResult {
    address: string;
    executionResult: IExecutionResult;
    transactionReceipt: {
        stateRoot: string;
        gasUsed: string;
        bloom: string;
        log: any[];
    };
}
export interface IGetTransactionRequest {
    /**
     * The transaction id
     */
    txid: string;
    /**
     * (optional, default=false) Whether to include watch-only addresses in balance calculation and details[]
     */
    include_watchonly?: boolean;
}
export interface IGetTransactionResult {
    amount: number;
    fee: number;
    confirmations: number;
    blockhash: string;
    blockindex: number;
    blocktime: number;
    txid: string;
    walletconflicts: any[];
    time: number;
    timereceived: number;
    "bip125-replaceable": "no" | "yes" | "unknown";
    details: any[];
    hex: string;
}
export declare class QtumRPC extends QtumRPCRaw {
    getInfo(): Promise<IGetInfoResult>;
    sendToContract(req: ISendToContractRequest): Promise<ISendToContractResult>;
    callContrct(req: ICallContractRequest): Promise<ICallContractResult>;
    getTransaction(req: IGetTransactionRequest): Promise<IGetTransactionResult>;
}
