"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import fetch from "isomorphic-fetch"
// import btoa from "btoa"
// import URL from "url-parse"
const fetch = require("isomorphic-fetch");
// const btoa = require("btoa")
const URL = require("url-parse");
class QtumRPCRaw {
    constructor(_baseURL) {
        this._baseURL = _baseURL;
        const url = new URL(_baseURL);
        this._origin = url.origin;
        this._reqHeaders = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        };
        if (url.username !== "" || url.password !== "") {
            const authToken = btoa(`${url.username}:${url.password}`);
            this._reqHeaders.Authorization = `Basic ${authToken}`;
        }
    }
    async rawCall(method, ...params) {
        const rpcCall = {
            method,
            params,
            id: this.idNonce++,
        };
        let res = await this.makeRPCCall(rpcCall);
        if (res.status === 402) {
            const auth = await res.json();
            res = await this.authCall(auth.id, rpcCall);
        }
        if (res.status === 401) {
            // body is empty
            throw new Error(await res.statusText);
        }
        // 404 if method doesn't exist
        if (res.status === 404) {
            throw new Error(`unknown method: ${method}`);
        }
        // 500 for other errors
        if (res.status === 500) {
            const error = await res.json();
            throw new Error(`[${error.code}] ${error.message}`);
        }
        const { result } = await res.json();
        return result;
    }
    makeRPCCall(rpcCall) {
        return fetch(`${this._origin}/`, {
            method: "POST",
            headers: this._reqHeaders,
            body: JSON.stringify(rpcCall),
        });
    }
    async authCall(authID, rpcCall) {
        // long-poll an authorization until its state changes
        const res = await fetch(`${this._origin}/api/authorizations/${authID}/onchange`);
        if (res.status !== 200) {
            const error = await res.json();
            throw new Error(error.message);
        }
        const auth = await res.json();
        if (auth.state === "denied") {
            throw new Error(`Authorization denied: ${authID}`);
        }
        if (auth.state === "accepted") {
            return this.makeRPCCall(Object.assign({}, rpcCall, { auth: auth.id }));
        }
    }
}
exports.QtumRPCRaw = QtumRPCRaw;
